###############################################
# FILE: domfuns2.pat
# SUBJ: New domain-specific functions.
# AUTH: AM
# CREATED: 31/Aug/02 17:17:59
# NOTE:	Generalized from a particular application.
###############################################

@DECL

########
# FUNC:   DOMADVL
# SUBJ:   Look for domain-relevant object in adverbial.
# CR:     08/06/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:
########

domadvl(L("advl"),L("sent"))
{
if (!L("advl"))
  return;
"dump.txt" << "[domadvl: " << pnvar(L("advl"),"$text") << "\n";
if (!pnvar(L("advl"),"dom object") )
  return;

# Copy to current _clause...
if (S("dom object"))
  return;
S("dom object") = pnvar(L("advl"),"dom object");
if (S("dom object"))
  return;
L("np") = pnvar(L("advl"),"nps");
if (!L("np"))
  {
  "dump.txt" << " [domadvl: no np.]" << "\n";
  return;
  }

S("dom object") = domnp(L("np"));
return;
}

########
# FUNC:   DOMINADVLS
# SUBJ:   Look for domain-relevant object in adverbials.
# CR:     08/06/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:
########

dominadvls(L("cnode"),L("name"))  # Clause node.
{
if (!L("cnode"))
  return 0;

if (L("adv") = pnvar(L("cnode"),"a1") )
  {
  if (L("obj") = pnvar(L("adv"),L("name")) )
    return L("adv");
  }

if (L("adv") = pnvar(L("cnode"),"a2") )
  {
  if (L("obj") = pnvar(L("adv"),L("name")) )
    return L("adv");
  }

if (L("adv") = pnvar(L("cnode"),"a3") )
  {
  if (L("obj") = pnvar(L("adv"),L("name")) )
    return L("adv");
  }

if (L("adv") = pnvar(L("cnode"),"a4") )
  {
  if (L("obj") = pnvar(L("adv"),L("name")) )
    return L("adv");
  }

if (L("adv") = pnvar(L("cnode"),"a5") )
  {
  if (L("obj") = pnvar(L("adv"),L("name")) )
    return L("adv");
  }

return 0;
}


########
# FUNC:   DOMNP
# SUBJ:   Look for domain-relevant object in np.
# CR:     08/20/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:	Can return things by placing them in the np itself...
########

domnp(L("np"),L("sent"))
{
if (!L("np") || !L("sent"))
  return 0;

if (!(L("sem") = pnvar(L("np"),"dom object") ))
  L("sem") = pnvar(L("np"),"sem");
if (!L("sem"))
  return 0;

# Do domain-specific processing here.
return 0;
}


########
# FUNC:   domnvj
# SUBJ:   Domain-specific semantics for np-vg-adj pattern.
# CR:     08/04/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE: These functions operate as soon as, eg, the nvj pattern
#	is identified in the parse tree.  It is dangerous at this
#	point to refer to clauses that come "before" or "after",
#	since clauses are collected in an indeterminate ordering by
#	the analyzer grammar.
#
#	Once clauses are collected, functions like clausetraverse()
#	can operate on the tree, after the dust has settled.  A
#   domain-specific
#	layering on some of those functions is needed as well...
########

domnvj(L("np"),L("vg"),L("adj"))
{

if (!L("np") || !L("adj"))
  return;

"dump.txt" << "[domnvj:]" << "\n";
#"dump.txt" << "np=" << pnvar(L("np"),"$text")
#	<< " vg=" << pnvar(L("vg"),"$text")
#	<< " adj=" << pnvar(L("adj"),"$text")
#	<< "\n";

L("np sem") = pnvar(L("np"),"sem");
if (!L("np sem"))
  L("np sem") = pnvar(L("np"),"stem");
if (!L("np sem"))
  L("np sem") = pnvar(L("np"),"$text");
if (!L("np sem"))
  return;

L("adj sem") = pnvar(L("adj"),"sem");
if (!L("adj sem"))
  L("adj sem") = pnvar(L("adj"),"stem");
if (!L("adj sem"))
  L("adj sem") = pnvar(L("adj"),"$text");
if (!L("adj sem"))
  return 0;

#L(adj val") = L("adj sem");
L("adj val") = pnvar(L("adj"),"$text");

"dump.txt" << "npsem=" << L("np sem")
	<< " adjsem=" << L("adj sem")
	<< "\n";

# Domain-specific processing here.
}

########
# FUNC:   domnvn
# SUBJ:   Semantics for np-vg-np active transitive pattern.
# CR:     08/05/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE: 
########

domnvn(L("np1"),L("vg"),L("np2"))
{

if (!L("np1") || !L("vg") || !L("np2"))
  return;

"dump.txt" << "[domnvn:]" << "\n";
"dump.txt" << "np1=" << pnvar(L("np1"),"$text")
	<< " vg=" << pnvar(L("vg"),"$text")
	<< " np2=" << pnvar(L("np2"),"$text")
	<< "\n";

L("np1 sem") = nodesem(L("np1"));
L("np2 sem") = nodesem(L("np2"));
L("vg sem") = nodesem(L("vg"));

# L("np2 val") = pnvar(L("np2"),"$text");
# L("np2 val") = pnvar(L("np2"),"$text");
L("np2 val") = nodestext(L("np2"), 0);

"dump.txt" << "np1sem=" << L("np1 sem")
    << " vgsem=" << L("vg sem")
	<< " np2sem=" << L("np2 sem")
	<< "\n";

# Looking for some patterns in hard-wired fashion.

if (L("np1 sem") == "there")
  {
  # "there was X" , or other copula...
  # ignoring things like "there emerges a trend..."
  
  # Examine np2.
  if (L("tmp") = pnvar(L("np2"),"dom object") )
  	{
	# Something like "retinal edema", implied domain object.
	L("np1 sem") = L("tmp");
	L("np1") = L("np2");
#	L("np2 val") = nodetext(L("np2"));

	# Get the text from np2 to the end of the clause.
	# Expands the fill text.	# 08/06/02 AM.
	# (In general, should store last node of clause in clause,
	# and watch out cause some np2 may come from anaphora and
	# other scary stuff.)
	L("np2 val") = nodestext(L("np2"), 0);
	}
  # Else "normal" np2.
  }
}

####################
####################
## SEMANTIC TRAVERSAL FNS (QCLAUSE100 PASS)
####################
####################

########
# FUNC:   DOMCLAUSEREGISTER
# SUBJ:   Manually handle a clause in the parse tree.
# CR:     08/05/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:	Quick and dirty.  Not building semantics in the KB,
#		for now, just in global variables.
########

domclauseregister(
	L("cnode"),	# Clause node.
	L("sent")	# Current sentence.
	)
{
if (!L("cnode") || !L("sent"))
  return;

"dump.txt"
	<< "[domclausereg: "
	<< pnvar(L("cnode"),"$text") << "]\n";

L("actor") = pnvar(L("cnode"),"actor");
if (!L("actor"))
  return;

L("actor sem") = nodesem(L("actor"));
"dump.txt" << 
	"[domclausereg: actor sem=" << L("actor sem") << "]\n";
L("side") = pnvar(L("actor"),"side");

# Reference resolution of sorts.
if (L("actor sem") == "thing")	# Or other semantically useless.
  {
  L("dom") = pnvar(L("cnode"),"dom object");
  if (L("dom"))
    {
    L("actor sem") == L("dom");
	pnreplaceval(L("actor"),"sem",L("dom"));
	}
  }

if (pnvar(L("cnode"),"pattern") == "nvn")
	# Handle nvn active transitive.
  {
  "dump.txt" << "nvn:\n";

  domnvn( L("actor"),
  			pnvar(L("cnode"),"act"),
			pnvar(L("cnode"),"dobj") );
  }
else if (pnvar(L("cnode"),"pattern") == "nvj")
  {
  "dump.txt" << "Call domnvj:\n";

  domnvj( L("actor"),
  			pnvar(L("cnode"),"act"),
			pnvar(L("cnode"),"adj role") );
  }
}
#############################################################
##     SEMANTIC FUNCTIONS
#############################################################

########
# FUNC:   DOMNEWEVENT
# SUBJ:   Application-specific fixups for new event.
# CR:     09/18/02 AM.
# INPUT:  
# OUTPUT: 
########

domnewevent(L("clause"),L("event"))
{
if (!L("clause") || !L("event"))
  return;

# Just transfer up some app slots.
domcopyattrs(L("clause"),L("event"));
}


########
# FUNC:   DOMMERGEEVENT
# SUBJ:   Application-specific merging of events.
# CR:     09/18/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:   Need to address app-specific conflicts in merging
#         events, may reject merge in some cases due to such.
########

dommergeevent(L("clause"),L("event"))
{
if (!L("clause") || !L("event"))
  return;

# Just transfer up some app slots.
domcopyattrs(L("clause"),L("event"));
}

########
# FUNC:   DOMCOPYVARS
# SUBJ:   Application-specific copy of node variables.
# CR:     09/19/02 AM.
########

domcopyvars(L("nfrom"),L("nto"))
{
if (!L("nfrom") || !L("nto"))
  return;
# Sample code for transerring domain info.
#if (L("tmp") = pnvar(L("nfrom"),"dom slot"))
#  pnreplaceval(L("nto"),"dom slot",L("tmp"));
#if (L("tmp") = pnvar(L("nfrom"),"dom val"))
#  pnreplaceval(L("nto"),"dom val",L("tmp"));
#if (L("tmp") = pnvar(L("nfrom"),"zone"))
#  pnreplaceval(L("nto"),"zone",L("tmp"));
#if (L("tmp") = pnvar(L("nfrom"),"dom type"))
#  pnreplaceval(L("nto"),"dom type",L("tmp"));
}

########
# FUNC:   DOMCOPYATTRS
# SUBJ:   Application-specific copy of concept attrs.
# CR:     09/20/02 AM.
########

domcopyattrs(L("nfrom"),L("nto"))
{
if (!L("nfrom") || !L("nto"))
  return;
#if (L("tmp") = strval(L("nfrom"),"dom slot"))
#  replaceval(L("nto"),"dom slot",L("tmp"));
#if (L("tmp") = strval(L("nfrom"),"dom val"))
#  replaceval(L("nto"),"dom val",L("tmp"));
#if (L("tmp") = strval(L("nfrom"),"dom type"))
#  replaceval(L("nto"),"dom type",L("tmp"));
#if (L("tmp") = strval(L("nfrom"),"zone"))
#  replaceval(L("nto"),"zone",L("tmp"));
}

########
# FUNC:   DOMCOPYNODETOCON
# SUBJ:   Application-specific copy from pnode to con.
# CR:     09/20/02 AM.
########

domcopynodetocon(L("nfrom"),L("conto"))
{
if (!L("nfrom") || !L("conto"))
  return;
#if (L("tmp") = pnvar(L("nfrom"),"dom val"))
#  replaceval(L("conto"),"dom val",L("tmp"));
#  replaceval(L("conto"),"dom os",L("tmp"));
#if (L("tmp") = pnvar(L("nfrom"),"dom type"))
#  replaceval(L("conto"),"dom type",L("tmp"));
#if (L("tmp") = pnvar(L("nfrom"),"zone"))
#  replaceval(L("conto"),"zone",L("tmp"));
}

########
# FUNC:   DOMCOMPOSE
# SUBJ:   Compose domain data from two nodes to suggested node.
# CR:     09/21/02 AM.
########

domcompose(L("n1"),L("n2"))
{
if (!L("n1") || !L("n2"))
  return;

L("sem1") = pnvar(L("n1"),"sem");
L("sem2") = pnvar(L("n2"),"sem");
if (L("sem1") && L("sem1") == L("sem2"))
  S("sem") = L("sem1");

# TODO: Merge domain-specific information from two nodes here.
# Sample code.
#L("val1") = pnvar(L("n1"),"dom val");
#L("val2") = pnvar(L("n2"),"dom val");
#if (L("val1") && L("val2"))
#	;	# Could put into multivalue variable.
#else if (L("val1"))
#  S("dom val") = L("val1");
#else if (L("val2"))
#  S("dom val") = L("val2");
#
#L("slot1") = pnvar(L("n1"),"dom slot");
#L("slot2") = pnvar(L("n2"),"dom slot");
#if (L("slot1") && L("slot2"))
#  ;
#else if (L("slot1"))
#  S("dom slot") = L("slot1");
#else if (L("slot2"))
#  S("dom slot") = L("slot2");
#
#L("typ1") = pnvar(L("n1"),"dom type");
#L("typ2") = pnvar(L("n2"),"dom type");
#if (L("typ1") && L("typ2"))
#  {
#  if (L("typ1") == L("typ2"))
#    S("dom type") = L("typ1");
#  else
#    S("dom type") = 0;
#  }
#else if (L("typ1"))
#  S("dom type") = L("typ1");
#else if (L("typ2"))
#  S("dom type") = L("typ2");
}

########
# FUNC:   DOMX
# SUBJ:   Compose domain data from two nodes.
# CR:     10/01/02 AM.
########

domx(L("n1"),L("n2"))
{
if (!L("n1") || !L("n2"))
  return;

# TODO:	Put your domain-specific merge code here.

}


########
# FUNC:   MAPDOMAIN
# SUBJ:   Traverse and convert events in kb to application-
#		  specific kb structures.
# CR:     09/17/02 AM.
# INPUT:  
# OUTPUT: 
########

mapdomain(L("events"))
{
"dump.txt" << "\n[mapdomain:]" << "\n";

if (!L("events"))
  return;

# Traverse the events gathered for the text.
#L("event") = down(L("events"));
#
#while (L("event"))
#  {
#  # Todo: Handle current event.
#  # Get next event.
#  L("event") = next(L("event"));
#  }
}

########
# FUNC:   DOMOUTPUT
# SUBJ:   Traverse findings hierarchy and output.
# CR:     09/20/02 AM.
# INPUT:  
# OUTPUT: 
########

domoutput()
{
# TODO:	Traverse domain-specific hierarchy in kb and produce
# output for it.
}

@@DECL
