###############################################
# FILE: messy.pat
# SUBJ: If cleaning up messy input
# AUTH: AM
# CREATED: 18/Mar/02 13:41:39
# MODIFIED: A preprocessing pass.
###############################################

@CODE
if (!G("messy"))
  exitpass();
G("lookupalpha") = 1;	# Flag that we've done lookup.
@@CODE

@NODES _LINE

# Sentinel.
@POST
  noop();
@RULES
_xNIL <-
	_xALPHA
	\-
	_xALPHA
	\-
	_xALPHA
	_xWILD [star match=( \- _xALPHA)]
	@@

#####
## CORRECTIONS TO DICTIONARY HERE.
#####

# 'past' is a prep.

# 'go' missing from dict!
@POST
  N("pos num") = 3;
  N("verb") = 95;	# Putting in some preferences!
  N("noun") = 4;
  N("adj") = 1;
  N("stem") = "go";
  N("pos") = "_verb";
@RULES
_xNIL <- go @@

# 'seek' listed as noun in dict, but can't corroborate.
@POST
  N("pos num") = 1;
  N("verb") = 100;
  N("stem") = "seek";
  N("pos") = "_verb";
@RULES
_xNIL <- seek @@
_xNIL <- seeks @@

# resolving listed as noun, can't corroborate.
@POST
  N("pos num") = 1;
  N("verb") = 1;
  N("stem") = "resolve";
  N("pos") = "_verb";
@RULES
_xNIL <- resolving @@

@POST
  N("pos num") = 2;
  N("verb") = 1;
  N("adv") = 1;		# MISSING.
  N("stem") = "please";
  N("pos") = "_verb";
@RULES
_xNIL <- please @@

# To avoid mis-stemming.
@POST
  N("pos num") = 1;
  N("noun") = 1;
  N("stem") = "thing";
  N("sem") = "thing";
  N("pos") = "_noun";
@RULES
_xNIL <- thing @@

@POST
  N("pos num") = 1;
  N("adj") = 1;
  N("stem") = "unirradiated";
  N("pos") = "_adj";
@RULES
_xNIL <- unirradiated @@

@POST
  N("pos num") = 2;
  N("adj") = 1;		# MISSING.
  N("noun") = 1;
  N("stem") = "miotic";
  N("pos") = "_adj";
@RULES
_xNIL <- miotic @@

@POST
# BUG: "do" is listed only as a noun in the dictionary!
  N("pos num") = 2;
  N("verb") = 1;	# MISSING.
  N("noun") = 1;
  N("stem") = "do";
  N("pos") = "_verb";
@RULES
_xNIL <- do @@

@POST
  N("pos num") = 2;
  N("verb") = 1;
  N("noun") = 1;	# MISSING.
  N("stem") = "whale";
  N("pos") = "_noun";
@RULES
_xNIL <- whaling @@

@POST
  N("pos num") = 2;
  N("verb") = 1;
  N("noun") = 1;
  N("stem") = "reprocess";
  N("pos") = "_noun";
@RULES
_xNIL <- reprocessing @@

@POST
  S("pos num") = 2;
  S("verb") = 1;
  S("noun") = 1;
  S("text") = "cochair";
  S("stem") = "cochair";
  S("pos") = "_noun";
  single();
@RULES
cochair <-
	co [s]
	\- [s]
	chair [s]
	@@

@POST
  N("pos num") = 2;
  N("verb") = 1;
  N("noun") = 1;
  N("stem") = "cochair";
  N("pos") = "_noun";
@RULES
_xNIL <- cochair @@

# Stemmer incorrectly does improving => improv!
@POST
  N("pos num") = 2;
  N("verb") = 1;
  N("adj") = 1;
  N("stem") = "improve";
  N("pos") = "_verb";
@RULES
_xNIL <- improving @@
_xNIL <- improved @@

@POST
  N("pos num") = 1;
  N("verb") = 1;
  N("stem") = "locate";
  N("pos") = "_verb";
@RULES
_xNIL <- locating @@

@POST
  N("pos num") = 3;
  N("noun") = 1;
  N("adj") = 1;
  N("sem") = "direction";
  N("stem") = strtolower(N("$text"));
  N("pos") = "_noun";
#  if (N("$length") == 1)
#    N("adv") = 50;	# confidence.
#  else
     N("adv") = 95;	# Ballworld domain.
@RULES
_xNIL <- _xWILD [one match=(
	n s e w
	ne se nw sw
	nne ene sse ese nnw wnw ssw wsw
	)] @@

#####
#####

# Looking for hyphenated junk that should be merged.
# alpha dash alpha
@CHECK
  S("text1") = N("$text",1);
  S("text2") = N("$text",3);
  S("txt") = S("text1") + S("text2");
  S("text") = strtolower(S("txt"));
  if (!spellword(S("text"))) # Combination must be a word.
     fail();
@POST
  # RETOKENIZE to create a merged word.
  # Todo: lookup the word and assign p-o-s if unambig, etc.
  group(1,3, S("txt"));
  group(1,1, "_retok");	# Flag these.	# 09/05/02 AM.
  N("text",1) = S("txt");	# Keep the merged text.
  N("lctext",1) = S("text");
  
  # Lookup in dictionary.		# 03/20/02 AM.
  lookupalpha(S("text"),N(1));	# 03/20/02 AM.
  N("tok type",1) = "alpha";	# Flag as alphabetic.
@RULES
_xNIL <-
	_xALPHA
	\-
	_xALPHA
	@@

# LOOKUP ALPHABETICS HERE.
# We're handling quirky ANSI chars that may be glommed onto
# a word, so may as well look up a word while we're at it,
# for efficiency.
# Todo: Need something that properly creates a new node, within
# the context of a rule match.  Need node for double-dash.
@CHECK
@POST
  S("txt") = N("$text");
  
  # Look for mangled alphabetic with double-dash char at end.
  if (strchr(S("txt"), ""))
  	{
	# Found one.  NLP++ needs a good way to insert a new node
	# into a list.
	S("len") = strlength(S("txt"));
	S("text") = strpiece(S("txt"),0,S("len")-2);
	group(1,1,S("text"));
	N("text") = S("text");
	N("double-dash") = 1; # Flag double-dash, for now.
	S("txt") = strtolower(S("text"));
    }
  else
    S("txt") = strtolower(S("txt"));

  # Look up in dictionary.
  lookupalpha(S("txt"),N(1));

  # Todo: May want to look up elsewhere...
  if (!N("pos num"))	# Not in dictionary.
  	{
	"unk.txt" << S("txt") << "\n";	# No suggestions.
#	"unknown.txt" << S("txt") << "\t";
#	if (strlength(S("txt")) < 4)
#	  "unknown.txt" << "\t";
#	if (strlength(S("txt")) < 8)
#	  "unknown.txt" << "\t";
#	if (strlength(S("txt")) < 12)
#	  "unknown.txt" << "\t";
#	if (S("corr") = spellcorrect(S("txt")) )
#	  "unknown.txt" << "suggest=" << S("corr");
#	"unknown.txt" << "\n";
	N("unknown") = 1;
	N("stem") = S("txt");
	unknownword(S("txt"),N(1));
	}
@RULES
_xNIL <- _xALPHA @@

# FOOTNOTES.
# Finding footnotes that merely appear as numbers interspersed
# in text will be a tedious problem, in general.
# Below are some quick and dirty finds of footnotes.
# A more general heuristic will examine each number and its
# context.  If it doesn't fold neatly into a noun phrase, then
# it becomes a candidate.
# (Note, this is really too early for this.  Need to find
# numbers that have a purpose, then examine what's left. Could
# do some counting schemes also -- make sure footnotes don't skip,
# etc.!)

#@POST
#  group(2,2,"_footnote");
#@RULES
#_xNIL <-
#	_xWHITE [star]
#	_xNUM
#	_xWHITE [star lookahead]
#	_xWILD [s one match=( \. \, and )]
#	@@


